import java.awt.BorderLayout;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

public class Algoritam extends JPanel {

	private List<Parametar> parametriAlgoritma;
	private JComboBox<String> comboBox;
	private List<Parametar> parametriGenotipa;
	private List<Parametar> parametriRegistrya;
	private String ime;
	private JPanel param;
	private String ruta;
	private JProgressBar progressBar;
	private int currentMaxEvals;
	private Projekt projekt;
	private String trenutniProblem;

	public Algoritam(List<Parametar> parametriAlgoritma, JComboBox<String> comboBox, String ime,
			JProgressBar progressBar, Projekt projekt) {
		this.parametriAlgoritma = parametriAlgoritma;
		this.comboBox = comboBox;
		this.ime = ime;
		this.progressBar = progressBar;
		this.projekt = projekt;

		setLayout(new BorderLayout());
	}

	public void setParametriGenotipa(List<Parametar> parametriGenotipa) {
		this.parametriGenotipa = parametriGenotipa;
	}

	public void setParametriRegistrya(List<Parametar> parametriRegistrya) {
		this.parametriRegistrya = parametriRegistrya;
		if (param != null) {
			promijeniParametre();
		}
	}
	
	public void setTrenutniProblem(String trenutniProblem) {
		this.trenutniProblem = trenutniProblem;
	}
	
	public void dodajParametre() {
		JPanel desno = new JPanel(new BorderLayout());

		desno.add(comboBox, BorderLayout.NORTH);

		param = new JPanel(new GridLayout(
				(parametriAlgoritma.size() + parametriGenotipa.size() + parametriRegistrya.size()) / 2, 2));
		desno.add(param, BorderLayout.CENTER);

		promijeniParametre();

		add(desno, BorderLayout.NORTH);

		JTextArea ulaz = new JTextArea();
		JTextArea izlaz = new JTextArea();
		JScrollPane ulazP = new JScrollPane(ulaz);
		JScrollPane izlazP = new JScrollPane(izlaz);

		ulaz.setBorder(BorderFactory.createLineBorder(Color.blue));
		izlaz.setBorder(BorderFactory.createLineBorder(Color.blue));

		ulaz.setText("Ulazna datoteka");
		ulaz.setEditable(false);
		izlaz.setText("Izlazna datoteka");
		izlaz.setEditable(false);

		JPanel prepanel = new JPanel(new BorderLayout());
		JPanel panel = new JPanel(new GridLayout(2, 1));

		JPanel startHelp = new JPanel(new FlowLayout());
		JButton start = new JButton("Izvedi");
		start.setPreferredSize(new Dimension(200, 30));
		startHelp.add(start);
		prepanel.add(panel, BorderLayout.CENTER);
		prepanel.add(startHelp, BorderLayout.SOUTH);
		panel.add(ulazP);
		panel.add(izlazP);

		start.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				start.setEnabled(false);
				update(0);
				StringBuilder sb = new StringBuilder();
				sb.append("<ECF>\n");
				sb.append("\t<Algorithm>\n");
				sb.append("\t\t<" + ime + ">\n");
				for (Parametar p : parametriAlgoritma) {
					sb.append("\t\t\t<Entry key =\"" + p.getXml() + "\">" + p.getVrijednost().getText() + "</Entry>\n");
				}
				sb.append("\t\t</" + ime + ">\n");
				sb.append("\t</Algorithm>\n");

				sb.append("\t<Genotype>\n");
				if (!trenutniProblem.equals("GPArtificialAnt")) {
					sb.append("\t\t<FloatingPoint>\n");
				} else {
					sb.append("\t\t<Tree>\n");
				}
				for (Parametar p : parametriGenotipa) {
					sb.append("\t\t\t<Entry key =\"" + p.getXml() + "\">" + p.getVrijednost().getText() + "</Entry>\n");
				}
				if (!trenutniProblem.equals("GPArtificialAnt")) {
					sb.append("\t\t</FloatingPoint>\n");
				} else {
					sb.append("\t\t</Tree>\n");
				}
				sb.append("\t</Genotype>\n");

				sb.append("\t<Registry>\n");
				for (Parametar p : parametriRegistrya) {
					if (p.getXml().equals("term.eval")) {
						currentMaxEvals = Integer.parseInt(p.getVrijednost().getText());
					}
					sb.append("\t\t<Entry key =\"" + p.getXml() + "\">" + p.getVrijednost().getText() + "</Entry>\n");
				}
				sb.append("\t</Registry>\n");

				sb.append("</ECF>\n");
				try {
					Path p = Paths.get("./param.txt");
					Files.write(p, sb.toString().getBytes());
				} catch (IOException e1) {
					System.out.println("GREŠKA PRVA!");
				}

				ProcessBuilder processB = new ProcessBuilder();
				processB.command(ruta, "./param.txt");
				Thread t = new Thread(() -> {
					Process process;
					try {
						ulaz.setText(sb.toString());
						process = processB.start();
						BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
						StringBuilder builder = new StringBuilder();
						String line = null;
						boolean check = false;
						
						while ((line = reader.readLine()) != null) {
							if (line.startsWith("Evaluations:")) {
								String[] dijelovi = line.split("\\s+");
								int evals = Integer.parseInt(dijelovi[1]);

								int value = (int) ((((double) evals) / currentMaxEvals) * 100);
								SwingUtilities.invokeLater(() -> update(value));
							}

							if (line.startsWith("Best of run:")) {
								check = true;
							}

							if (check) {
								builder.append(line);
								builder.append(System.getProperty("line.separator"));
							}
						}
						SwingUtilities.invokeLater(() -> update(100));
						String result = builder.toString();
						
						Path p = Paths.get("./out.txt");
						Files.write(p, result.getBytes());

						izlaz.setText(result);
						start.setEnabled(true);
					} catch (Exception e2) {
					}

				});
				
				t.start();
			}
		});
		add(prepanel, BorderLayout.CENTER);
	}
	
	private Object update(int value) {
		progressBar.setValue(value);
		progressBar.setString(value + "%");

		return null;
	}
	private void promijeniParametre() {
		param.removeAll();
		for (Parametar p : parametriAlgoritma) {
			param.add(p);
		}
		for (Parametar p : parametriGenotipa) {
			param.add(p);
		}
		for (Parametar p : parametriRegistrya) {
			param.add(p);
		}
	}

	public void setRuta(String ruta) {
		this.ruta = ruta;
	}
}
