import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

public class Projekt extends JFrame {

	private static final int startingWidth = 1000;
	private static final int startingHeight = 500;
	private static final int startingPositionX = 100;
	private static final int startingPositionY = 100;
	private String rutaFunctionMin;
	private String rutaCOCO;
	private String rutaGPArtificialAnt;
	
	private JTabbedPane tabbedPane;
	private JProgressBar progressBar;

	public Projekt(String ruta) {
		rutaFunctionMin = ruta + "/examples/FunctionMin/VS/Debug/FunctionMin.exe";
		rutaCOCO = ruta + "/examples/COCO/VS/Debug/COCO.exe";
		rutaGPArtificialAnt = ruta + "/examples/GPArtificialAnt/VS/Debug/GPArtificialAnt.exe";
		setSize(startingWidth, startingHeight);
		setLocation(startingPositionX, startingPositionY);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setTitle("EA u ECF-u");

		initGUI();
	}

	public static void main(String[] args) {
		if (args.length != 1) {
			throw new IllegalArgumentException("Potrebno je unijeti jedan argument, putanju do ECF-a na računalu.");
		}
		SwingUtilities.invokeLater(() -> new Projekt(args[0]).setVisible(true));
	}

	private void initGUI() {
		Container cp = getContentPane();
		cp.setLayout(new BorderLayout());
		progressBar = new JProgressBar();
		
		tabbedPane = new JTabbedPane();
		cp.add(tabbedPane, BorderLayout.CENTER);
		
		//Prvi
		List<Parametar> parametriAlgoritmaSteadyStateTournament = new ArrayList<>();
		Parametar p1 = new Parametar(new JLabel("Veličina turnira"), new JTextField("3"), "tsize");
		parametriAlgoritmaSteadyStateTournament.add(p1);
		createNewTab(parametriAlgoritmaSteadyStateTournament, "SteadyStateTournament", tabbedPane);
		
		//Drugi
		List<Parametar> parametriAlgoritmaGeneticAnnealing = new ArrayList<>();
		Parametar p2 = new Parametar(new JLabel("Faktor hlađenja"), new JTextField("0.7"), "coolingfactor");
		Parametar p3 = new Parametar(new JLabel("Početna veličina energetske banke"), new JTextField("200"), "energybank");
		Parametar p4 = new Parametar(new JLabel("Elitizam"), new JTextField("0"), "elitism");
		parametriAlgoritmaGeneticAnnealing.add(p2);
		parametriAlgoritmaGeneticAnnealing.add(p3);
		parametriAlgoritmaGeneticAnnealing.add(p4);
		createNewTab(parametriAlgoritmaGeneticAnnealing, "GeneticAnnealing", tabbedPane);
			
		//Treci
		List<Parametar> parametriAlgoritmaElimination = new ArrayList<>();
		Parametar p5 = new Parametar(new JLabel("Selekcijski pritisak"), new JTextField("10"), "selpressure");
		Parametar p6 = new Parametar(new JLabel("Postotak populacije za eliminirat"), new JTextField("0.6"), "gengap");
		parametriAlgoritmaElimination.add(p5);
		parametriAlgoritmaElimination.add(p6);
		createNewTab(parametriAlgoritmaElimination, "Elimination", tabbedPane);
		
		JPanel doljnji = new JPanel(new FlowLayout());
		progressBar.setStringPainted(true);
		progressBar.setPreferredSize(new Dimension(400, 30));

		doljnji.add(progressBar);
		cp.add(doljnji, BorderLayout.SOUTH);

	}

	private void createNewTab(List<Parametar> parametriAlgoritma, String string, JTabbedPane tabbedPane) {
		JComboBox<String> comboBox = new JComboBox<>();
		
		//FMin
		List<Parametar> parametriGenotipaFunctionMin = new ArrayList<>();
		Parametar p1 = new Parametar(new JLabel("Doljnja grancia"), new JTextField("-50"), "lbound");
		Parametar p2 = new Parametar(new JLabel("Gornja grancia"), new JTextField("50"), "ubound");
		Parametar p3 = new Parametar(new JLabel("Broj dimenzija"), new JTextField("10"), "dimension");
		parametriGenotipaFunctionMin.add(p1);
		parametriGenotipaFunctionMin.add(p2);
		parametriGenotipaFunctionMin.add(p3);
		List<Parametar> parametriRegistryaFunctionMin = new ArrayList<>();
		Parametar p4 = new Parametar(new JLabel("funkcija"), new JTextField("1"), "function");
		Parametar p5 = new Parametar(new JLabel("Veličina populacije"), new JTextField("500"), "population.size");
		Parametar p6 = new Parametar(new JLabel("Vjerojatnost mutacije"), new JTextField("0.3"), "mutation.indprob");
		Parametar p7 = new Parametar(new JLabel("Najveći broj generacija"), new JTextField("500"), "term.maxgen");
		Parametar p8 = new Parametar(new JLabel("Fitness za prekid"), new JTextField("1e-12"), "term.fitnessval");
		Parametar p9 = new Parametar(new JLabel("Najveći broj evalucaija"), new JTextField("100000"), "term.eval");
		parametriRegistryaFunctionMin.add(p4);
		parametriRegistryaFunctionMin.add(p5);
		parametriRegistryaFunctionMin.add(p6);
		parametriRegistryaFunctionMin.add(p7);
		parametriRegistryaFunctionMin.add(p8);
		parametriRegistryaFunctionMin.add(p9);
		
		//COCO
		List<Parametar> parametriGenotipaCOCO = new ArrayList<>();
		Parametar p10 = new Parametar(new JLabel("Doljnja grancia"), new JTextField("-5"), "lbound");
		Parametar p11 = new Parametar(new JLabel("Gornja grancia"), new JTextField("5"), "ubound");
		Parametar p12 = new Parametar(new JLabel("Broj dimenzija"), new JTextField("10"), "dimension");
		parametriGenotipaCOCO.add(p10);
		parametriGenotipaCOCO.add(p11);
		parametriGenotipaCOCO.add(p12);
		List<Parametar> parametriRegistryaCOCO = new ArrayList<>();
		Parametar p13 = new Parametar(new JLabel("COCO funkcija"), new JTextField("1"), "coco.function");
		Parametar p14 = new Parametar(new JLabel("COCO instanca"), new JTextField("1"), "coco.instance");
		Parametar p15 = new Parametar(new JLabel("Veličina populacije"), new JTextField("100"), "population.size");
		Parametar p16 = new Parametar(new JLabel("Vjerojatnost mutacije"), new JTextField("0.3"), "mutation.indprob");
		Parametar p17 = new Parametar(new JLabel("Najveći broj generacija"), new JTextField("9999"), "term.maxgen");
		Parametar p18 = new Parametar(new JLabel("Najveći broj evalucaija"), new JTextField("100000"), "term.eval");
		parametriRegistryaCOCO.add(p13);
		parametriRegistryaCOCO.add(p14);
		parametriRegistryaCOCO.add(p15);
		parametriRegistryaCOCO.add(p16);
		parametriRegistryaCOCO.add(p17);
		parametriRegistryaCOCO.add(p18);
		
		//GPArtificialAnt
		List<Parametar> parametriGenotipaGPArtificialAnt = new ArrayList<>();
		Parametar p19 = new Parametar(new JLabel("Najveća dubina"), new JTextField("4"), "maxdepth");
		Parametar p20 = new Parametar(new JLabel("Najmanja dubina"), new JTextField("1"), "mindepth");
		Parametar p21 = new Parametar(new JLabel("Funkcijski set"), new JTextField("p2 p3 ifa"), "functionset");
		Parametar p22 = new Parametar(new JLabel("Terminalni set"), new JTextField("left right move"), "terminalset");
		parametriGenotipaGPArtificialAnt.add(p19);
		parametriGenotipaGPArtificialAnt.add(p20);
		parametriGenotipaGPArtificialAnt.add(p21);
		parametriGenotipaGPArtificialAnt.add(p22);
		List<Parametar> parametriRegistryaGPArtificialAnt = new ArrayList<>();
		Parametar p23 = new Parametar(new JLabel("Datoteka s rutama za učenje"), new JTextField("learning_trails.txt"), "learning_trails");
		Parametar p24 = new Parametar(new JLabel("Datoteka s testnim rutama"), new JTextField("test_trails.txt"), "test_trails");
		Parametar p25 = new Parametar(new JLabel("Veličina populacije"), new JTextField("500"), "population.size");
		Parametar p26 = new Parametar(new JLabel("Najveći broj generacija"), new JTextField("50"), "term.maxgen");
		Parametar p27 = new Parametar(new JLabel("Najveći broj evalucaija"), new JTextField("25000"), "term.eval");
		parametriRegistryaGPArtificialAnt.add(p23);
		parametriRegistryaGPArtificialAnt.add(p24);
		parametriRegistryaGPArtificialAnt.add(p25);
		parametriRegistryaGPArtificialAnt.add(p26);
		parametriRegistryaGPArtificialAnt.add(p27);
		
		Algoritam alg = new Algoritam(parametriAlgoritma, comboBox, string, progressBar, this);
		comboBox.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				JComboBox<String> comboBox = (JComboBox) e.getSource();
				String selected = (String) comboBox.getSelectedItem();
				
				if (selected.equals("COCO")) {
					alg.setParametriGenotipa(parametriGenotipaCOCO);
					alg.setParametriRegistrya(parametriRegistryaCOCO);
					alg.setRuta(rutaCOCO);
					alg.setTrenutniProblem("COCO");
				} else if (selected.equals("FunctionMin")) {
					alg.setParametriGenotipa(parametriGenotipaFunctionMin);
					alg.setParametriRegistrya(parametriRegistryaFunctionMin);
					alg.setRuta(rutaFunctionMin);
					alg.setTrenutniProblem("FunctionMin");
				} else {
					alg.setParametriGenotipa(parametriGenotipaGPArtificialAnt);
					alg.setParametriRegistrya(parametriRegistryaGPArtificialAnt);
					alg.setRuta(rutaGPArtificialAnt);
					alg.setTrenutniProblem("GPArtificialAnt");
				}
				
				Projekt.this.repaint();
			}
		});
		comboBox.addItem("COCO");
		comboBox.addItem("FunctionMin");
		comboBox.addItem("GPArtificialAnt");
		alg.dodajParametre();
		tabbedPane.add(string, alg);
	}
}
